"use strict";

const React = require("react");
const TableComponent = require("../components/table");
const Table = TableComponent.Table;
const TableCell = TableComponent.TableCell;
const TableRow = TableComponent.TableRow;
const StatePersistedMixin = require("../components/util").StatePersistedMixin;
const CsvLink = require("./subscription-matching-util").CsvLink;

const Messages = React.createClass({
  mixins: [StatePersistedMixin],

  rowComparator: function(a, b, columnIndex, ascending) {
    const orderCondition = ascending ? 1 : -1;
    const aValue = a.props.message;
    const bValue = b.props.message;
    return aValue.toLowerCase().localeCompare(bValue.toLowerCase()) * orderCondition;
  },

  buildRows: function(rawMessages, systems) {
    const result = rawMessages.map(function(rawMessage, index) {
      const data = rawMessage["data"];
      var message;
      var additionalInformation;
      switch(rawMessage["type"]) {
        case "unknownPartNumber" :
          message = t("Unsupported part number detected");
          additionalInformation = data["partNumber"];
          break;
        case "physicalGuest" :
          message = t("Physical system is reported as virtual guest, please check hardware data");
          additionalInformation = systems[data["id"]].name;
          break;
        case "guestWithUnknownHost" :
          message = t("Virtual guest has unknown host, assuming it is a physical system");
          additionalInformation = systems[data["id"]].name;
          break;
        case "unknownCpuCount" :
          message = t("System has an unknown number of sockets, assuming 16");
          additionalInformation = systems[data["id"]].name;
          break;
        default:
          message = rawMessage["type"];
          additionalInformation = data;
      }
      const columns = [
        <TableCell key="message" content={message} />,
        <TableCell key="additionalInformation" content={additionalInformation} />
      ];
      return <TableRow key={index} columns={columns} message={message} />;
    });
    return result;
  },

  render: function() {
    var body;
    if (this.props.messages.length > 0) {
      body = (
        <div>
          <p>{t("Please review warning and information messages below.")}</p>
          <Table
            headers={[t("Message"), t("Additional information")]}
            rows={this.buildRows(this.props.messages, this.props.systems)}
            loadState={this.props.loadState}
            saveState={this.props.saveState}
            rowComparator={this.rowComparator}
            sortableColumnIndexes={[0]}
          />
          <CsvLink name="message_report.csv" />
        </div>
      );
    }
    else {
      body = <p>{t("No messages from the last match run.")}</p>
    }

    return (
      <div className="row col-md-12">
        <h2>{t("Messages")}</h2>
        {body}
      </div>
    );
  }
});

module.exports = {
  Messages: Messages,
}
