"use strict";

const React = require("react");
const TableComponent = require("../components/table");
const Table = TableComponent.Table;
const TableCell = TableComponent.TableCell;
const TableRow = TableComponent.TableRow;
const StatePersistedMixin = require("../components/util").StatePersistedMixin;
const PopUp = require("../components/popup").PopUp;
const UtilComponent =  require("./subscription-matching-util");
const StrongText = UtilComponent.StrongText;
const SystemLabel = UtilComponent.SystemLabel;
const ToolTip = UtilComponent.ToolTip;
const humanReadablePolicy = UtilComponent.humanReadablePolicy;
const WarningIcon =  require("./subscription-matching-util").WarningIcon;
const Network = require("../utils/network");

const Pins = React.createClass({
  mixins: [StatePersistedMixin],

  getInitialState: function() {
    return {showPopUp: false};
  },

  rowComparator: function(a, b, columnIndex, ascending) {
    const aRaw = a.props["rawData"];
    const bRaw = b.props["rawData"];
    const columnKeyInRawData=["systemName", "subscriptionDescription", "subscriptionPolicy", "subscriptionEndDate", "subscriptionPartNumber", "status"];
    const columnKey = columnKeyInRawData[columnIndex];
    const orderCondition = ascending ? 1 : -1;

    const aValue = aRaw[columnKey];
    const bValue = bRaw[columnKey];

    var result = aValue.localeCompare(bValue);
    if (result == 0) {
      const aId = aRaw["id"];
      const bId = bRaw["id"];
      result = aId > bId ? 1 : (aId < bId ? -1 : 0);
    }

    return result * orderCondition;
  },

  buildRows: function(pins, systems, subscriptions, onClickAction) {
    return pins.map((p) => {
      const system = systems[p.systemId];
      const systemName = system == null ? "System " + p.systemId : system.name;
      const systemType = system == null ? null : system.type;
      const subscription = subscriptions[p.subscriptionId];
      const subscriptionDescription = subscription == null ? "Subscription " + p.subscriptionId : subscription.description;
      const subscriptionPolicy = subscription == null ? " " : subscription.policy;
      const subscriptionEndDate = subscription == null ? " " : subscription.endDate;
      const subscriptionPartNumber = subscription == null ? "" : subscription.partNumber;
      const columns = [
        <TableCell key="systemId" content={<SystemLabel id={p.systemId} name={systemName} type={systemType} />} />,
        <TableCell key="subscription" content={subscriptionDescription} />,
        <TableCell key="policy" content={humanReadablePolicy(subscriptionPolicy)} />,
        <TableCell key="endDate" content={
          <ToolTip content={moment(subscriptionEndDate).fromNow()}
            title={moment(subscriptionEndDate).format("LL")} />}
        />,
        <TableCell key="partNumber" content={subscriptionPartNumber} />,
        <TableCell key="status" content={<PinStatus status={p.status} />} />,
        <TableCell key="button" content={
          <PinButton
            onClick={() => onClickAction(p.id)}
            content={<span><i className="fa fa-trash-o"></i>{t("Delete Pin")}</span>}
           />
          }
        />
      ];
      const rawData = {
        id: p.id,
        systemName: systemName,
        subscriptionDescription: subscriptionDescription,
        subscriptionPolicy: subscriptionPolicy,
        subscriptionEndDate: subscriptionEndDate,
        subscriptionPartNumber: subscriptionPartNumber,
        status: p.status
     };
      return <TableRow key={p.id} columns={columns} rawData={rawData} />
    });
  },

  onRemovePin: function(pinId) {
    Network.post("/rhn/manager/subscription-matching/pins/"+pinId+"/delete")
      .promise.then(data => this.props.onPinChanged(data));
  },

  showPopUp: function() {
    this.setState({showPopUp: true});
  },

  closePopUp: function() {
    this.setState({showPopUp: false});
  },

  savePin: function(systemId, subscriptionId) {
    Network.post("/rhn/manager/subscription-matching/pins", {system_id: systemId, subscription_id: subscriptionId})
      .promise.then(data => this.props.onPinChanged(data));
    $("#addPinPopUp").modal("hide"); //to trigger popup close action
    this.closePopUp();
  },

  render: function() {
    const popUpContent = this.state.showPopUp ? <AddPinPopUp products={this.props.products} systems={this.props.systems} subscriptions={this.props.subscriptions} onSavePin={this.savePin} /> : null;
    return (
      <div className="row col-md-12">
        <h2>{t("Pins")}</h2>
        <p>
          {t("You can pin a subscription to a system to suggest a certain association to the matching algorithm. ")}
          <br />
          {t("Next time a matching is attempted, the algorithm will try to produce a result that applies the subscription to the system you specified. ")}
          <br />
          {t("Note that the algorithm might determine that a certain pin cannot be respected, ")}
          {t("depending on a subscription's availablility and applicability rules, in that case it will be shown as not satisfied. ")}
        </p>

        {this.props.pinnedMatches.length > 0 ?
          <Table headers={[t("System"), t("Subscription"), t("Policy"), t("End date"), t("Part number"), t("Status"), t("")]}
            rows={this.buildRows(this.props.pinnedMatches, this.props.systems, this.props.subscriptions, this.onRemovePin)}
            loadState={() => this.state["table"]}
            saveState={(state) => {this.state["table"] = state;}}
            rowComparator={this.rowComparator}
            sortableColumnIndexes={[0, 1, 2, 3, 4, 5]}
          /> :
          <p>{t("No pins defined. You can create one with the button below.")}</p>}

        <button type="button" className="btn btn-primary" onClick={this.showPopUp} data-toggle="modal" data-target="#addPinPopUp">
          <i className="fa fa-plus"></i>{t("Add a Pin")}
        </button>
        <PopUp
          title={t("Add a Pin")}
          className="modal-lg"
          id="addPinPopUp"
          content={popUpContent}
          onClosePopUp={this.closePopUp}
        />
      </div>
    );
  }
});

const PinStatus = (props) => {
  if (props.status == "pending") {
    return <span><i className="fa fa-hourglass-start"></i><em>{t("pending next run")}</em></span>;
  }
  if (props.status == "satisfied") {
    return <span><i className="fa fa-check text-success"></i>{t("satisfied")}</span>;
  }
  return <span><WarningIcon />{t("not satisfied")}</span>;
}

const PinButton = (props) =>
  <button className="btn btn-default btn-cell" onClick={props.onClick}>
    {props.content}
  </button>
;

const AddPinPopUp = React.createClass({
  getInitialState:function() {
    return {systemId: null};
  },

  rowComparator: function(a, b, columnIndex, ascending) {
    const aRaw = a.props["rawData"];
    const bRaw = b.props["rawData"];
    const columnKeyInRawData=["name", "cpuCount"];
    const columnKey = columnKeyInRawData[columnIndex];
    const orderCondition = ascending ? 1 : -1;

    const aValue = aRaw[columnKey];
    const bValue = bRaw[columnKey];

    var result = 0;
    if (columnKey == "name") {
      result = aValue.localeCompare(bValue);
    }
    if (columnKey == "cpuCount") {
      result = aValue - bValue;
    }

    if (result == 0) {
      const aId = aRaw["id"];
      const bId = bRaw["id"];
      result = aId - bId;
    }

    return result * orderCondition;
  },

  buildRows: function() {
    return Object.keys(this.props.systems).map((id) => {
      const s = this.props.systems[id];
      const columns = [
        <TableCell key="system" content={<SystemLabel id={id} name={s.name} type={s.type} />} />,
        <TableCell key="cpuCount" content={s.cpuCount} />,
        <ProductTableCell key="products" products={this.props.products} productIds={s.productIds} />,
        <TableCell key="button" content={
          <PinButton
            onClick={() => this.onSystemSelected(id)}
            content={<span>{t("Select")} <i className="fa fa-arrow-right fa-right"></i></span>}
          />}
        />
      ];
      return <TableRow key={id} columns={columns} rawData={s} />
    });
  },

  onBackClicked: function () {
    this.setState({systemId: null});
  },

  onSystemSelected:function(systemId) {
    this.setState({systemId: systemId});
  },

  onSubscriptionSelected: function(subscriptionId) {
    this.props.onSavePin(this.state.systemId, subscriptionId);
  },

  render: function() {
    var popUpContent;
    if (this.state.systemId == null) {
      popUpContent = (
        <div>
          <h4 className="add-pin-popup-subtitle">{t("Available Systems")}</h4>
          <p>{t("Step 1/2: select the system to pin from the table below.")}</p>
          <Table headers={[t("System"), t("Socket/IFL count"), t("Products"), t("")]} rows={this.buildRows()}
            rowFilter={(tableRow, searchValue) => tableRow.props["rawData"]["name"].toLowerCase().indexOf(searchValue.toLowerCase()) > -1}
            filterPlaceholder={t("Filter by name")}
            sortableColumnIndexes={[0, 1]}
            rowComparator={this.rowComparator}
          />
        </div>
      );
    }
    else {
      const system = this.props.systems[this.state.systemId];
      popUpContent = (
        <div>
          <h4 className="add-pin-popup-subtitle">{t("Available Subscriptions for the Selected System")}</h4>
          <p>{t("Step 2/2: pick a subscription for system ")}<strong>{system.name}</strong></p>
          <PinSubscriptionSelector onSubscriptionSelected={this.onSubscriptionSelected}
            subscriptions={system.possibleSubscriptionIds.map(
            p => this.props.subscriptions[p]
          )} />
          <p>
            <button className="btn btn-default" onClick={this.onBackClicked}>
              <i className="fa fa-arrow-left"></i>
              {t("Back to sytem selection")}
            </button>
          </p>
        </div>
      );
    }
    return (popUpContent);
  }
});

const ProductTableCell = (props) => {
  const productLength = props.productIds.length;

  if (productLength == 0){
    return <TableCell />;
  }

  const firstProductName = props.products[props.productIds[0]].productName;
  if (productLength == 1) {
    return <TableCell content={firstProductName} />;
  }

  const productNames = props.productIds
    .map(i => props.products[i].productName)
    .reduce((previousValue, currentValue) => previousValue + ", " + currentValue)
  ;
  return (
    <TableCell content={
      <ToolTip
        content={firstProductName + ", ..."}
        title={productNames}
      />}
    />
  );
};

const PinSubscriptionSelector = React.createClass({
  buildRows: function(possibleSubscriptions, onClickAction) {
    return possibleSubscriptions.map((s) => {
      const columns = [
        <TableCell key="partNumber" content={s.partNumber} />,
        <TableCell key="description" content={s.description} />,
        <TableCell key="policy" content={humanReadablePolicy(s.policy)} />,
        <TableCell key="endDate" content={
          <ToolTip content={moment(s.endDate).fromNow()}
            title={moment(s.endDate).format("LL")} />}
        />,
        <TableCell key="button" content={
          <PinButton
            onClick={() => onClickAction(s.id)}
            content={<span><i className="fa fa-map-pin"></i>{t("Save Pin")}</span>}
          />}
        />
      ];
      return <TableRow key={s.id} columns={columns} rawData={s} />
    });
  },

  render: function() {
    if (this.props.subscriptions.length > 0) {
      return <Table headers={[t("Part number"),t("Description"), t("Policy"), t("End date"), t("")]}
          rows={
            this.buildRows(
              this.props.subscriptions,
              this.props.onSubscriptionSelected)
            }
        />;
    }
    else {
      return <p>{t("No subscriptions have been found to match this system, considering all products installed, either directly or in virtual guests.")}</p>
    }
  },
});

module.exports = {
  Pins: Pins,
}
